@@ -6,21 +6,33 @@
  *
  */
 #include "eim.h"
+#include <array>
 #include <chewing.h>
 #include <cstdarg>
 #include <cstdio>
+#include <fcitx-config/iniparser.h>
+#include <fcitx-utils/capabilityflags.h>
+#include <fcitx-utils/key.h>
 #include <fcitx-utils/keysym.h>
 #include <fcitx-utils/log.h>
+#include <fcitx-utils/standardpaths.h>
+#include <fcitx-utils/textformatflags.h>
 #include <fcitx-utils/utf8.h>
+#include <fcitx/addoninstance.h>
 #include <fcitx/candidatelist.h>
 #include <fcitx/event.h>
 #include <fcitx/inputcontext.h>
 #include <fcitx/inputpanel.h>
+#include <fcitx/instance.h>
 #include <fcitx/statusarea.h>
 #include <fcitx/text.h>
 #include <fcitx/userinterface.h>
 #include <fcitx/userinterfacemanager.h>
+#include <filesystem>
 #include <memory>
+#include <stdexcept>
+#include <string>
+#include <string_view>
 #include <utility>
 #include <vector>
 
@@ -33,12 +45,17 @@ namespace {
 
 constexpr int CHEWING_MAX_LEN = 18;
 
-const char *builtin_selectkeys[] = {
-    "1234567890", "asdfghjkl;", "asdfzxcv89", "asdfjkl789",
-    "aoeuhtn789", "1234qweras", "dstnaeo789",
-};
+constexpr auto builtin_selectkeys = std::to_array<std::string_view>({
+    "1234567890",
+    "asdfghjkl;",
+    "asdfzxcv89",
+    "asdfjkl789",
+    "aoeuhtn789",
+    "1234qweras",
+    "dstnaeo789",
+});
 
-static_assert(FCITX_ARRAY_SIZE(builtin_selectkeys) ==
+static_assert(builtin_selectkeys.size() ==
                   ChewingSelectionKeyI18NAnnotation::enumLength,
               "Enum mismatch");
 
@@ -64,7 +81,7 @@ public:
     void select(InputContext *inputContext) const override {
         auto *ctx = engine_->context();
         auto pageSize = chewing_get_candPerPage(ctx);
-        int page = index_ / pageSize + chewing_cand_CurrentPage(ctx);
+        int page = (index_ / pageSize) + chewing_cand_CurrentPage(ctx);
         int off = index_ % pageSize;
         if (page < 0 || page >= chewing_cand_TotalPage(ctx)) {
             return;
@@ -274,11 +291,11 @@ void logger(void * /*context*/, int /*level*/, const char *fmt, ...) {
 } // namespace
 
 ChewingContext *getChewingContext() {
-    const auto &sp = StandardPath::global();
-    std::string dictData =
-        sp.locate(StandardPath::Type::Data, "libchewing/dictionary.dat");
+    const auto &sp = StandardPaths::global();
+    std::filesystem::path dictData =
+        sp.locate(StandardPathsType::Data, "libchewing/dictionary.dat");
     if (!dictData.empty()) {
-        std::string sysPath = fs::dirName(dictData);
+        std::string sysPath = dictData.parent_path().string();
         return chewing_new2(sysPath.c_str(), nullptr, nullptr, nullptr);
     }
     return chewing_new();
@@ -334,6 +351,7 @@ void ChewingEngine::doReset(InputContextEvent &event) {
     chewing_cand_close(ctx);
     chewing_clean_preedit_buf(ctx);
     chewing_clean_bopomofo_buf(ctx);
+    chewing_Reset(ctx);
     updateUI(event.inputContext());
 }
 
@@ -365,7 +383,7 @@ void ChewingEngine::deactivate(const InputMethodEntry &entry,
     }
 }
 
-bool ChewingEngine::handleCandidateKeyEvent(const KeyEvent &keyEvent) {
+bool ChewingEngine::handleCandidateKeyEvent(const KeyEvent &keyEvent) const {
     auto *ic = keyEvent.inputContext();
     auto candidateList = std::dynamic_pointer_cast<ChewingCandidateList>(
         ic->inputPanel().candidateList());
@@ -452,10 +470,11 @@ void ChewingEngine::keyEvent(const InputMethodEntry &entry,
         return;
     }
 
+    int chewingReturnValue = 0;
     if (keyEvent.key().check(FcitxKey_space)) {
-        chewing_handle_Space(ctx);
+        chewingReturnValue = chewing_handle_Space(ctx);
     } else if (keyEvent.key().check(FcitxKey_Tab)) {
-        chewing_handle_Tab(ctx);
+        chewingReturnValue = chewing_handle_Tab(ctx);
     } else if (keyEvent.key().isSimple()) {
         if (keyEvent.rawKey().states().test(KeyState::Shift)) {
             chewing_set_easySymbolInput(ctx, *config_.EasySymbolInput ? 1 : 0);
@@ -470,14 +489,14 @@ void ChewingEngine::keyEvent(const InputMethodEntry &entry,
                 return;
             }
         }
-        chewing_handle_Default(ctx, scan_code);
+        chewingReturnValue = chewing_handle_Default(ctx, scan_code);
         chewing_set_easySymbolInput(ctx, 0);
     } else if (keyEvent.key().check(FcitxKey_BackSpace)) {
         if ((chewing_buffer_Check(ctx)) == 0 &&
             (chewing_bopomofo_Check(ctx) == 0)) {
             return;
         }
-        chewing_handle_Backspace(ctx);
+        chewingReturnValue = chewing_handle_Backspace(ctx);
         if ((chewing_buffer_Check(ctx)) == 0 &&
             (chewing_bopomofo_Check(ctx) == 0)) {
             keyEvent.filterAndAccept();
@@ -485,13 +504,13 @@ void ChewingEngine::keyEvent(const InputMethodEntry &entry,
             return;
         }
     } else if (keyEvent.key().check(FcitxKey_Escape)) {
-        chewing_handle_Esc(ctx);
+        chewingReturnValue = chewing_handle_Esc(ctx);
     } else if (keyEvent.key().check(FcitxKey_Delete)) {
         if ((chewing_buffer_Check(ctx)) == 0 &&
             (chewing_bopomofo_Check(ctx) == 0)) {
             return;
         }
-        chewing_handle_Del(ctx);
+        chewingReturnValue = chewing_handle_Del(ctx);
         if ((chewing_buffer_Check(ctx)) == 0 &&
             (chewing_bopomofo_Check(ctx) == 0)) {
             keyEvent.filterAndAccept();
@@ -499,48 +518,51 @@ void ChewingEngine::keyEvent(const InputMethodEntry &entry,
             return;
         }
     } else if (keyEvent.key().check(FcitxKey_Up)) {
-        chewing_handle_Up(ctx);
+        chewingReturnValue = chewing_handle_Up(ctx);
     } else if (keyEvent.key().check(FcitxKey_Down)) {
-        chewing_handle_Down(ctx);
+        chewingReturnValue = chewing_handle_Down(ctx);
     } else if (keyEvent.key().check(FcitxKey_Page_Down)) {
-        chewing_handle_PageDown(ctx);
+        chewingReturnValue = chewing_handle_PageDown(ctx);
     } else if (keyEvent.key().check(FcitxKey_Page_Up)) {
-        chewing_handle_PageUp(ctx);
+        chewingReturnValue = chewing_handle_PageUp(ctx);
     } else if (keyEvent.key().check(FcitxKey_Right)) {
-        chewing_handle_Right(ctx);
+        chewingReturnValue = chewing_handle_Right(ctx);
     } else if (keyEvent.key().check(FcitxKey_Left)) {
-        chewing_handle_Left(ctx);
+        chewingReturnValue = chewing_handle_Left(ctx);
     } else if (keyEvent.key().check(FcitxKey_Home)) {
-        chewing_handle_Home(ctx);
+        chewingReturnValue = chewing_handle_Home(ctx);
     } else if (keyEvent.key().check(FcitxKey_End)) {
-        chewing_handle_End(ctx);
+        chewingReturnValue = chewing_handle_End(ctx);
     } else if (keyEvent.key().check(FcitxKey_space, KeyState::Shift)) {
-        chewing_handle_ShiftSpace(ctx);
+        chewingReturnValue = chewing_handle_ShiftSpace(ctx);
     } else if (keyEvent.key().check(FcitxKey_Left, KeyState::Shift)) {
-        chewing_handle_ShiftLeft(ctx);
+        chewingReturnValue = chewing_handle_ShiftLeft(ctx);
     } else if (keyEvent.key().check(FcitxKey_Right, KeyState::Shift)) {
-        chewing_handle_ShiftRight(ctx);
+        chewingReturnValue = chewing_handle_ShiftRight(ctx);
     } else if (keyEvent.key().check(FcitxKey_Return)) {
-        chewing_handle_Enter(ctx);
+        chewingReturnValue = chewing_handle_Enter(ctx);
     } else if (keyEvent.key().states() == KeyState::Ctrl &&
                Key(keyEvent.key().sym()).isDigit()) {
-        chewing_handle_CtrlNum(ctx, keyEvent.key().sym());
+        chewingReturnValue = chewing_handle_CtrlNum(ctx, keyEvent.key().sym());
     } else {
         // to do: more chewing_handle
         return;
     }
 
+    CHEWING_DEBUG() << "Chewing return value: " << chewingReturnValue;
     if (chewing_keystroke_CheckIgnore(ctx)) {
+        CHEWING_DEBUG() << "Keystroke ignored";
         return;
     }
     if (chewing_keystroke_CheckAbsorb(ctx)) {
+        CHEWING_DEBUG() << "Keystroke absorbed";
         keyEvent.filterAndAccept();
     }
     if (chewing_commit_Check(ctx)) {
         keyEvent.filterAndAccept();
         ic->commitString(safeChewing_commit_String(ctx));
     }
-    return updateUI(ic);
+    updateUI(ic);
 }
 
 void ChewingEngine::filterKey(const InputMethodEntry & /*entry*/,
@@ -554,7 +576,8 @@ void ChewingEngine::filterKey(const InputMethodEntry & /*entry*/,
          keyEvent.key().check(FcitxKey_space, KeyState::Shift) ||
          keyEvent.key().check(FcitxKey_Tab) ||
          keyEvent.key().check(FcitxKey_Return, KeyState::Shift))) {
-        return keyEvent.filterAndAccept();
+        keyEvent.filterAndAccept();
+        return;
     }
 
     if (!ic->inputPanel().candidateList()) {
